/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Handling process instances        */
/*                                                                    */
/* This example shows what can be done with process instances queried */
/* from the execution service. 					      */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>		        // character classification
#include <time.h>			// let's do the time warp again


int main (int argc, char ** argv)
{

  vector<FmcjProcessInstance>		iList;

  unsigned short			year		= 0;
  unsigned short			month		= 0;
  unsigned short			day		= 0;
  unsigned short			hour		= 0;
  unsigned short			minute		= 0;
  unsigned short			second		= 0;
  
  unsigned long                       	threshold    	= 20;
  unsigned long                       	i       	= 0;
  unsigned long				j		= 0;


  string 				PIListSort	="NAME ASC";

  string 				choice		="";
  int					usrchoice	= 0;

  time_t                           	ltime          	= 0;

  FmcjExecutionService  		service;
  
  
/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    cout << "\nUsage: " << argv[0] << " <UserID> <Password>" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();
  rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }
  cout << "Logged on with UserID '" << service.UserID() 
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;


/*--------------------------------------------------------------------*/
/* Display current instances list, query user for action              */
/*--------------------------------------------------------------------*/  

  do 					// Beginning of main tool loop
  {
    // Clear instance vector

    iList.erase(iList.begin(),iList.end());

    // Display as many process instances as specified with threshold

    cout << "\nAvailable process instances:\n"
         << endl;
  
    rc = service.QueryProcessInstances( FmcjNoFilter,
     				         &PIListSort,
  				         &threshold,
  				         iList );

    if (rc != FMC_OK)
    {
      cout << "Querying process instances failed with RC " << rc << " - " 
           << result->MessageText() << endl;
      service.Logoff();
      // Deinit API environment
      FmcjGlobal::Disconnect();
      delete(result);
      return -1;
    }

    cout << "#   State        Instance name" << endl;
    cout << "------------------------------" << endl;
  
    if (iList.size() > 0 )
    {
      for( i=0; i< iList.size(); i++ )
      {
        cout << i;
        if (i<10) cout << " ";
        cout << "  ";
        switch (iList[i].State())
        {
         case FmcjProcessInstance::undefined: cout << "undefined  "; break;
         case FmcjProcessInstance::ready: cout << "ready      "; break;
         case FmcjProcessInstance::running: cout << "running    "; break;
         case FmcjProcessInstance::finished: cout << "finished   "; break;
         case FmcjProcessInstance::terminated: cout << "terminated "; break;
         case FmcjProcessInstance::suspended: cout << "suspended  "; break;
         case FmcjProcessInstance::terminating: cout << "terminating"; break;
         case FmcjProcessInstance::suspending: cout << "suspending "; break;
         case FmcjProcessInstance::deleted: cout << "deleted    "; break;
        }
        cout << "  " << iList[i].Name() << endl;
      }
    }     
    else cout << "  <no instances available>" << endl;

    cout << endl
         << "Commands: [v] View instance information             [t] Terminate instance\n"
         << "          [s] Suspend instance                      [n] Start instance\n"
         << "          [r] Resume instance\n"
         << "          [a] Restart instance                      [q] Quit program\n"
         << "[v/s/r/a/t/n/q]? ";
    cout.flush();
  
    cin >> choice;

    usrchoice=0;
  
    if (strncmp(choice.c_str(),"v",1)==0) usrchoice=1;
    if (strncmp(choice.c_str(),"s",1)==0) usrchoice=2;
    if (strncmp(choice.c_str(),"r",1)==0) usrchoice=3;
    if (strncmp(choice.c_str(),"a",1)==0) usrchoice=4;
    if (strncmp(choice.c_str(),"t",1)==0) usrchoice=5;
    if (strncmp(choice.c_str(),"n",1)==0) usrchoice=6;
    if (strncmp(choice.c_str(),"q",1)==0) usrchoice=255;

    if (iList.size()>0)
    switch (usrchoice)
    {

      case 1:
/*-------------------------------------------------------------------*/
/* Show some instance properties                                     */
/*-------------------------------------------------------------------*/  

    	do
    	{
  	  cout << "Please enter instance number: ";
    	  cout.flush();
    	  cin >> choice;
    	  j=0;
    	  for (i=0; i<strlen(choice.c_str()); i++)
    	  { if (isdigit(choice.c_str()[i])) j++; }
    	} while (i!=j);
    	
    	i = atoi(choice.c_str());
	cout << endl;
	
	if (i>(iList.size()-1) && iList.size()>0)
	{
	  cout << "Please use range 0-" << iList.size()-1 << "." << endl;
	}
	else
	{
	  cout << "Instance properties" << endl;
	  cout << "-------------------" << endl;
	  cout << "   Name     : " << iList[i].Name() << endl;
	  cout << "   Object id: " << iList[i].PersistentOid() << endl;
	  cout << "   Suspension expiration time: " 
	       << string(iList[i].SuspensionExpirationTime()) << endl << endl;
        }

	break;


      case 2:
/*-------------------------------------------------------------------*/
/* Demonstrates how to suspend a process instance.                   */
/*-------------------------------------------------------------------*/  

    	do
    	{
  	  cout << "Please enter instance number: ";
    	  cout.flush();
    	  cin >> choice;
    	  j=0;
    	  for (i=0; i<strlen(choice.c_str()); i++)
    	  { if (isdigit(choice.c_str()[i])) j++; }
    	} while (i!=j);
    	
    	i = atoi(choice.c_str());
	cout << endl;
	
	if (i>(iList.size()-1) && iList.size()>0)
	{
	  cout << "Please use range 0-" << iList.size()-1 << "." << endl;
	}
	else				// user's choice is in range
	{
          time(&ltime);	    
          cout << "Current time: " << ctime(&ltime)
               << "\nDo you want to set a suspension expiration time? [y/n] ";
	  cout.flush();
	    
	  cin >> choice;
	    
	  if (strncmp(choice.c_str(),"y",1)==0)
	  {
 	    cout << "Enter year  : "; cout.flush(); cin >> year;
	    cout << "Enter month : "; cout.flush(); cin >> month;
	    cout << "Enter day   : "; cout.flush(); cin >> day;
	    cout << "Enter hour  : "; cout.flush(); cin >> hour;
	    cout << "Enter minute: "; cout.flush(); cin >> minute;
	      
	    FmcjDateTime * qTime = new FmcjDateTime(year, month, day, hour, minute, second);
	      
	    // Suspend the process instance and all subordinated processes
	    // and let them be resumed automatically at the specified time.
 	   
 	    rc = iList[i].Suspend( *qTime, true );
 	    
 	    delete(qTime);
	  }
	  else
	  {
	    // Suspend the process instance and all subordinated processes

 	    rc = iList[i].Suspend( true );
	  }

	  if (rc != FMC_OK)
  	  {
    	    cout << "Suspending instance failed with RC " << rc << " - " 
         	   << result->MessageText() << endl << endl;
	  }

        } // end else: user's choice is in range

        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Demonstrates how to resume a process instance.                    */
/*-------------------------------------------------------------------*/  

    	do
    	{
  	  cout << "Please enter instance number: ";
    	  cout.flush();
    	  cin >> choice;
    	  j=0;
    	  for (i=0; i<strlen(choice.c_str()); i++)
    	  { if (isdigit(choice.c_str()[i])) j++; }
    	} while (i!=j);
    	
    	i = atoi(choice.c_str());
	cout << endl;
	
	if (i>(iList.size()-1) && iList.size()>0)
	{
	  cout << "Please use range 0-" << iList.size()-1 << "." << endl;
	}
	else
	{
	  // Resume the process instance and all subordinated processes
 	  
 	  rc = iList[i].Resume( true );

	  if (rc != FMC_OK)
  	  {
    	    cout << "Resuming instance failed with RC " << rc << " - " 
         	 << result->MessageText() << endl << endl;
	  }

        }

        break;


      case 4:
/*-------------------------------------------------------------------*/
/* Demonstrates how to restart a process instance.                   */
/*-------------------------------------------------------------------*/  

    	do
    	{
  	  cout << "Please enter instance number: ";
    	  cout.flush();
    	  cin >> choice;
    	  j=0;
    	  for (i=0; i<strlen(choice.c_str()); i++)
    	  { if (isdigit(choice.c_str()[i])) j++; }
    	} while (i!=j);
    	
    	i = atoi(choice.c_str());
	cout << endl;
	
	if (i>(iList.size()-1) && iList.size()>0)
	{
	  cout << "Please use range 0-" << iList.size()-1 << "." << endl;
	}
	else
	{
 	  rc = iList[i].Restart();

	  if (rc != FMC_OK)
  	  {
    	    cout << "Restarting instance failed with RC " << rc << " - " 
         	 << result->MessageText() << endl << endl;
	  }

        }

        break;


      case 5:
/*-------------------------------------------------------------------*/
/* Demonstrates how to terminate a process instance.                 */
/*-------------------------------------------------------------------*/  

    	do
    	{
  	  cout << "Please enter instance number: ";
    	  cout.flush();
    	  cin >> choice;
    	  j=0;
    	  for (i=0; i<strlen(choice.c_str()); i++)
    	  { if (isdigit(choice.c_str()[i])) j++; }
    	} while (i!=j);
    	
    	i = atoi(choice.c_str());
	cout << endl;
	
	if (i>(iList.size()-1) && iList.size()>0)
	{
	  cout << "Please use range 0-" << iList.size()-1 << "." << endl;
	}
	else
	{
 	  rc = iList[i].Terminate();

	  if (rc != FMC_OK)
  	  {
    	    cout << "Terminating instance failed with RC " << rc << " - " 
         	 << result->MessageText() << endl << endl;
	  }

        }

        break;


      case 6:
/*-------------------------------------------------------------------*/
/* Demonstrates how to start a process instance.                     */
/*-------------------------------------------------------------------*/  
        
    	do
    	{
  	  cout << "Please enter instance number: ";
    	  cout.flush();
    	  cin >> choice;
    	  j=0;
    	  for (i=0; i<strlen(choice.c_str()); i++)
    	  { if (isdigit(choice.c_str()[i])) j++; }
    	} while (i!=j);
    	
    	i = atoi(choice.c_str());
	cout << endl;
	
	if (i>(iList.size()-1) && iList.size()>0)
	{
	  cout << "Please use range 0-" << iList.size()-1 << "." << endl;
	}
	else
	{
	  // Start instance without passing an input container

 	  rc = iList[i].Start();

	  if (rc != FMC_OK)
  	  {
    	    cout << "Starting instance failed with RC " << rc << " - " 
         	 << result->MessageText() << endl << endl;
	  }

        }

        break;

      }  				// end of switch
      else cout << endl << "No instances available!" << endl;
  					
  } while ( (usrchoice!=255) );		// end of main tool loop


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/  

  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
