/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Tutorial - Monitoring processes              */
/*                                                                    */
/* This sample program shows how to monitor a process with its        */
/* subordinated programs, blocks and subprocesses                     */
/**********************************************************************/

#include <fmcjcrun.h>                   /* MQWF C API interface       */
#include <ctype.h>                      /* character classification   */
#include <stdio.h>                      /* for standard in-output     */
#include <stdlib.h>                     /*                            */
#include <string.h>                     /* string conversion          */
#define INFOSIZE 4097

int main (int argc, char ** argv)
{
  FmcjExecutionServiceHandle                service          = 0;

  FmcjProcessInstanceHandle                 processInstance  = 0;
  FmcjProcessInstanceVectorHandle           viList           = 0;


  FmcjActivityInstanceHandle                activityInstance = 0;
  FmcjActivityInstanceVectorHandle          aList            = 0;

  FmcjControlConnectorInstanceHandle        ccInstance       = 0;
  FmcjControlConnectorInstanceVectorHandle  ccList           = 0;

  FmcjPointHandle                           point            = 0;
  FmcjPointVectorHandle                     pointList        = 0;

  FmcjInstanceMonitorHandle                 piMonlevel0      = 0;
  FmcjInstanceMonitorHandle                 piMonlevel1      = 0;
  FmcjInstanceMonitorHandle                 biMon            = 0;

  FmcjResultHandle                          result           = 0;
  FmcjCDateTime                             timeValue;

  unsigned long                             vectSize         = 0;
  unsigned long                             vectSizeAList    = 0;
  unsigned long                             vectSizeCCList   = 0;

  APIRET                                    rc               = FMC_OK;

  char                                      choice[1]        = "";
  char                                      tInfo[INFOSIZE]  = "";

  int                                       usrchoice        = 0;
  int                                       i                = 0;
  int                                       j                = 0;
  int                                       aiIndex          = 0;
  int                                       cciIndex         = 0;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    printf("\nUsage: %s %s %s\n\n",argv[0],"<UserID>","<Password>");
    return -1;
  }
   fflush(stdout);
  rc = FmcjGlobalConnect();
  result = FmcjResultObjectOfCurrentThread();

  rc = FmcjExecutionServiceAllocate( &service );
  if (rc != FMC_OK)
  {
    printf("Service object could not be allocated: %u\n",rc);
    FmcjGlobalDisconnect();
    return -1;
  }
  rc = FmcjExecutionServiceLogon(service, argv[1], argv[2],
                                 Fmc_SM_Default,Fmc_SA_NotSet );

  if (rc != FMC_OK)
  {
    printf("Logon failed with RC %u %s %s\n",rc," - ",
                      FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    /* Deinit API environment */
    rc = FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();

    return -1;
  }
  printf("\nLogged on with UserID '%s' ",
             FmcjExecutionServiceUserID(service, tInfo, INFOSIZE));
  printf("on system '%s' ",
             FmcjExecutionServiceSystemName(service, tInfo, INFOSIZE));
  printf("at system group '%s'.\n",
             FmcjExecutionServiceSystemGroupName(service, tInfo, INFOSIZE));
  fflush(stdout);


/*-------------------------------------------------------------------*/
/* Query available process instances                                 */
/*-------------------------------------------------------------------*/

  do                                    // Beginning of main loop
  {
    rc = FmcjExecutionServiceQueryProcessInstances(service,
                                                   FmcjNoFilter,
                                                               "NAME ASC",
                                                                   FmcjNoThreshold,
                                                               &viList );
    if (rc != FMC_OK)
    {
      printf("Querying process instances failed with RC %u%s%s\n",
             rc," - ",FmcjResultMessageText(result, tInfo, INFOSIZE));
      fflush(stdout);

      FmcjProcessInstanceVectorDeallocate(&viList);
      FmcjExecutionServiceLogoff(service);
      /* Deinit API environment */
      rc= FmcjExecutionServiceDeallocate( &service );
      FmcjGlobalDisconnect();
      return -1;
    }
    printf(" #  State        Instance name \n" );
    printf("------------------------------ \n" );
    fflush(stdout);

    vectSize = FmcjProcessInstanceVectorSize(viList);
    if (vectSize > 0)
    {
      for( i=0; i< vectSize; i++ )
      {
        if (i < 10) printf(" %u%s",i,"  ");
        else        printf("%u%s",i,"  ");

        processInstance = FmcjProcessInstanceVectorNextElement(viList);

        switch (FmcjProcessInstanceState(processInstance))
        {
          case Fmc_PS_NotSet:      printf("undefined   "); break;
          case Fmc_PS_Ready:       printf("ready       "); break;
          case Fmc_PS_Running:     printf("running     "); break;
          case Fmc_PS_Finished:    printf("finished    "); break;
          case Fmc_PS_Terminated:  printf("terminated  "); break;
          case Fmc_PS_Suspended:   printf("suspended   "); break;
          case Fmc_PS_Terminating: printf("terminating "); break;
          case Fmc_PS_Suspending:  printf("suspending  "); break;
          case Fmc_PS_Deleted:     printf("deleted     "); break;
       }

       printf(" %s\n",FmcjProcessInstanceName(processInstance,tInfo,INFOSIZE));
       fflush(stdout);

       rc = FmcjProcessInstanceDeallocate(&processInstance);
      }
    }
    else printf("  <no instances available>\n");
    do
    {
      if ((vectSize-1) < 100)
        printf("\nPlease enter the instance number you want to view the monitor of: (0-%u%s",vectSize-1,") ");
      else
        printf("\nPlease enter the instance number you want to view the monitor of: (1-99)");
      fflush(stdout);

      scanf("%2[^\n]",tInfo);
      j=0;
      for (i=0; i < strlen(tInfo); i++)
      { if (isdigit(tInfo[i])) j++; }
      if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
    } while (i!=j);
    i = atoi(tInfo);
    fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
    printf("\n");
    fflush(stdout);
    if (i > (vectSize-1))
    {
      printf("Please use range 0-%u%s\n",vectSize-1,".");
    }
    else    /* user's choice is in range */
    {
      /* Get the selected instance */
      processInstance = FmcjProcessInstanceVectorFirstElement(viList);
      for (j = 0; j < i; j++)
      {
        rc = FmcjProcessInstanceDeallocate(&processInstance);
        processInstance = FmcjProcessInstanceVectorNextElement(viList);
      }

      /* Obtain monitor of selected process instance */
      rc = FmcjProcessInstanceObtainProcessMonitor(processInstance,
                                            false,
                                            &piMonlevel0 );
      if (rc != FMC_OK)
      {
        printf("Obtaining monitor failed with RC %u%s%s\n",
               rc," - ",FmcjResultMessageText(result, tInfo, INFOSIZE));
        fflush(stdout);

        FmcjProcessInstanceDeallocate(&processInstance);
        FmcjProcessInstanceVectorDeallocate(&viList);
        FmcjExecutionServiceLogoff(service);
        /* Deinit API environment */
        rc= FmcjExecutionServiceDeallocate( &service );
        FmcjGlobalDisconnect();
        return -1;
      }

      /* Show all activity instances and control connector instances */

      aList = FmcjInstanceMonitorActivityInstances(piMonlevel0);

      ccList = FmcjInstanceMonitorControlConnectorInstances(piMonlevel0);

      printf("\n#   Type       Activity instance name\n");
      printf("-------------------------------------\n");

      /* Show activity instances                      */

      vectSizeAList = FmcjActivityInstanceVectorSize(aList);
      if (vectSizeAList > 0 )
      {
        for(aiIndex=0; aiIndex < vectSizeAList; aiIndex++ )
        {
          activityInstance = FmcjActivityInstanceVectorNextElement(aList);

          if (aiIndex < 10) printf(" %u%s",aiIndex,"  ");
          else              printf("%u%s",aiIndex,"  ");

          switch (FmcjActivityInstanceKind(activityInstance))
          {
           case Fmc_AT_NotSet:  printf("undefined"); break;
           case Fmc_AT_Process: printf("process  "); break;
           case Fmc_AT_Program: printf("program  "); break;
           case Fmc_AT_Block:   printf("block    "); break;
          }
          printf("  %s\n",
                 FmcjActivityInstanceName(activityInstance,tInfo,INFOSIZE));
          fflush(stdout);

          rc = FmcjActivityInstanceDeallocate(&activityInstance);
        }
      }
      else printf("  <no instances available>\n");

      fflush(stdout);
      /* Show control connector instances */

      printf("\n#   State          Control connector instance name\n");
      printf("--------------------------------------------------\n");

      vectSizeCCList = FmcjControlConnectorInstanceVectorSize(ccList);
      if (vectSizeCCList > 0 )
      {
        for(cciIndex=aiIndex; cciIndex < vectSizeCCList+aiIndex; cciIndex++ )
        {
          ccInstance = FmcjControlConnectorInstanceVectorNextElement(ccList);

          if (cciIndex < 10) printf(" %u%s",cciIndex,"  ");
          else               printf("%u%s",cciIndex,"  ");

          switch (FmcjControlConnectorInstanceState(ccInstance))
          {
           case Fmc_CS_False:        printf("false        "); break;
           case Fmc_CS_True:         printf("true         "); break;
           case Fmc_CS_NotEvaluated: printf("not evaluated"); break;
           case Fmc_CS_NotSet:       printf("not set      "); break;
          }
          if (!FmcjControlConnectorInstanceNameIsNull(ccInstance))
               printf("  \n",
                      FmcjControlConnectorInstanceName(ccInstance,tInfo,INFOSIZE));

          else printf("  <not set>\n");

          rc = FmcjControlConnectorInstanceDeallocate(&ccInstance);
        }
        cciIndex--;
      }
      else printf("  <no instances available>\n");

      fflush(stdout);

      do
      {
        if ((cciIndex) < 100)
          printf("\nWhich activity/control connector instance\ndo you want to examine closer? (0-%u%s",cciIndex,") ");
        else
          printf("\nWhich activity/control connector instance\ndo you want to examine closer? (1-99)");
        fflush(stdout);

        scanf("%2[^\n]",tInfo);
        j=0;
        for (i=0; i < strlen(tInfo); i++)
        { if (isdigit(tInfo[i])) j++; }
        if (i!=j) fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
      } while (i!=j);
      i = atoi(tInfo);
      fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
      printf("\n");
      fflush(stdout);
      if (i > cciIndex || i < 0 )
      {
        printf("Please use range 0-%u%s\n",cciIndex,".");
      }
      else    /* user's choice is in range */
      {
        if (i < aiIndex)
        {
          /* Examine activity instance closer */

          /* Get the selected instance */
          activityInstance = FmcjActivityInstanceVectorFirstElement(aList);
          for (j = 0; j < i; j++)
          {
            rc = FmcjActivityInstanceDeallocate(&activityInstance);
            activityInstance = FmcjActivityInstanceVectorNextElement(aList);
          }

          switch (FmcjActivityInstanceKind(activityInstance))
          {
            case Fmc_AT_NotSet:
                  printf("\nBecause of an unknown reason, this activity instance has not been initialised.\n");   break;

            case Fmc_AT_Program:

                  /* Showing some program information */

                  printf("\nActivity instance - program information\n");
                  printf("---------------------------------------\n");
              printf("State          : ");
              switch (FmcjActivityInstanceState(activityInstance))
              {
                case Fmc_AS_NotSet:         printf("undefined    "); break;
                case Fmc_AS_Ready:          printf("ready        "); break;
                case Fmc_AS_Running:        printf("running      "); break;
                case Fmc_AS_Finished:       printf("finished     "); break;
                case Fmc_AS_Terminated:     printf("terminated   "); break;
                case Fmc_AS_Suspended:      printf("suspended    "); break;
                case Fmc_AS_Inactive:       printf("inactive     "); break;
                case Fmc_AS_CheckedOut:     printf("checkedOut   "); break;
                case Fmc_AS_InError:        printf("inError      "); break;
                case Fmc_AS_Executed:       printf("executed     "); break;
                case Fmc_AS_Planning:       printf("planning     "); break;
                case Fmc_AS_ForceFinished:  printf("forceFinished"); break;
                case Fmc_AS_Skipped:        printf("skipped      "); break;
                case Fmc_AS_Deleted:        printf("deleted      "); break;
                case Fmc_AS_Terminating:    printf("terminating  "); break;
                case Fmc_AS_Suspending:     printf("suspending   "); break;
              }
               printf("\nImplementation : %s\n",
                      FmcjActivityInstanceImplementation(activityInstance,tInfo,INFOSIZE));

               timeValue = FmcjActivityInstanceActivationTime(activityInstance);
               printf("Activation time: %s\n",
                    FmcjDateTimeAsString(&timeValue, tInfo, INFOSIZE));
               fflush(stdout);
               break;

            case Fmc_AT_Process:

              /* Fetch the corresponding process instance monitor */

              printf("\nSubprocess instance monitor information\n");
              printf("---------------------------------------\n");

              piMonlevel1 = FmcjInstanceMonitorObtainProcessMonitor
                            (piMonlevel0, activityInstance, false);
              rc = FmcjResultRc(result);
              if (rc == FMC_ERROR_DOES_NOT_EXIST)
              {
                printf("   A process instance of this subprocess does not exist yet,\n");
                printf("   hence it can't be monitored!\n");
              }
              else if (rc != FMC_OK)
              {
                printf("Obtaining monitor failed with RC %u-%s\n",
                        rc,FmcjResultMessageText(result, tInfo, INFOSIZE));
                fflush(stdout);

                FmcjInstanceMonitorDeallocate(&piMonlevel0);
                FmcjInstanceMonitorDeallocate(&piMonlevel1);

                FmcjActivityInstanceDeallocate(&activityInstance);
                FmcjActivityInstanceVectorDeallocate(&aList);

                FmcjControlConnectorInstanceVectorDeallocate(&ccList);

                FmcjProcessInstanceDeallocate(&processInstance);
                FmcjProcessInstanceVectorDeallocate(&viList);

                FmcjExecutionServiceLogoff(service);
                /* Deinit API environment */
                rc= FmcjExecutionServiceDeallocate( &service );
                FmcjGlobalDisconnect();
                return -1;
              }
              else
              {

                    /* Clear vectors */

                FmcjActivityInstanceVectorDeallocate(&aList);
                FmcjControlConnectorInstanceVectorDeallocate(&ccList);

                aList = FmcjInstanceMonitorActivityInstances(piMonlevel1);
                ccList = FmcjInstanceMonitorControlConnectorInstances(piMonlevel1);

                printf("Number of subordinate activity instances         : %u\n",
                     FmcjActivityInstanceVectorSize(aList));
                printf("Number of subordinate control connector instances: %u\n",
                     FmcjControlConnectorInstanceVectorSize(ccList));
                fflush(stdout);
              }
              FmcjInstanceMonitorDeallocate(&piMonlevel1);

              break;

            case Fmc_AT_Block:

              /* Fetch the corresponding block instance monitor */

              printf("\nBlock instance monitor information\n");
              printf("----------------------------------\n");

              biMon = FmcjInstanceMonitorObtainBlockMonitor
                                           (piMonlevel0, activityInstance);
              rc = FmcjResultRc(result);
              if (rc != FMC_OK)
              {
                printf("Obtaining monitor failed with RC %u-%s\n",
                        rc,FmcjResultMessageText(result, tInfo, INFOSIZE));
                fflush(stdout);

                FmcjInstanceMonitorDeallocate(&piMonlevel0);
                /*FmcjBlockInstanceMonitorDeallocate(&biMon);*/

                FmcjActivityInstanceDeallocate(&activityInstance);
                FmcjActivityInstanceVectorDeallocate(&aList);

                FmcjControlConnectorInstanceVectorDeallocate(&ccList);

                FmcjProcessInstanceDeallocate(&processInstance);
                FmcjProcessInstanceVectorDeallocate(&viList);

                FmcjExecutionServiceLogoff(service);
                /* Deinit API environment */
                rc= FmcjExecutionServiceDeallocate( &service );
                FmcjGlobalDisconnect();
                return -1;
              }

              /* Clear vectors */

              FmcjActivityInstanceVectorDeallocate(&aList);
              FmcjControlConnectorInstanceVectorDeallocate(&ccList);

              aList = FmcjInstanceMonitorActivityInstances(biMon);
              ccList = FmcjInstanceMonitorControlConnectorInstances(biMon);

              printf("Number of subordinate activity instances         : %u\n",
                   FmcjActivityInstanceVectorSize(aList));
              printf("Number of subordinate control connector instances: %u\n",
                   FmcjControlConnectorInstanceVectorSize(ccList));

              fflush(stdout);
              /*FmcjBlockInstanceMonitorDeallocate(&biMon);*/
              break;
          }
          FmcjActivityInstanceDeallocate(&activityInstance);
        }


        else
        {
          /* Examine control connector instance closer */

          /* Get the selected instance */
          ccInstance = FmcjControlConnectorInstanceVectorFirstElement(ccList);
          for (j = 0; j < (i-aiIndex) ; j++)
          {
            rc = FmcjControlConnectorInstanceDeallocate(&ccInstance);
            ccInstance = FmcjControlConnectorInstanceVectorNextElement(ccList);
          }


          printf("\nControl connector instance information\n");
          printf("--------------------------------------\n");
          printf("Source activity Oid : %s\n",
                 FmcjControlConnectorInstancePersistentOidOfSourceActivity(ccInstance,tInfo,INFOSIZE));
          printf("Target activity Oid : %s\n",
                 FmcjControlConnectorInstancePersistentOidOfTargetActivity(ccInstance,tInfo,INFOSIZE));
          printf("Transition Condition: %s\n",
                 FmcjControlConnectorInstanceTransitionCondition(ccInstance,tInfo,INFOSIZE));
          printf("Bend points:\n");

          pointList = FmcjControlConnectorInstanceBendPoints(ccInstance);

          vectSize = FmcjPointVectorSize(pointList);
          if (vectSize > 0)
          {
            for (j=0; j < vectSize; j++)
            {
              point = FmcjPointVectorNextElement(pointList);
              printf("   [%u] X: %u", j, FmcjPointXPosition(point));
              printf(" Y: %u\n", FmcjPointYPosition(point));

              FmcjPointDeallocate(&point);
            }
          }
          else
          {
            printf("   <no bend points exist>\n");
          }
          fflush(stdout);
          FmcjPointVectorDeallocate(&pointList);
        }

      }

      FmcjActivityInstanceVectorDeallocate(&aList);
      FmcjControlConnectorInstanceVectorDeallocate(&ccList);
      FmcjInstanceMonitorDeallocate(&piMonlevel0);
      FmcjProcessInstanceDeallocate(&processInstance);

    } /* end else */

    printf("\nDo you want to restart or quit [r/q]? ");
    fflush(stdout);
    fgets(choice,3,stdin);

    if (strncmp(choice,"q",1)==0) usrchoice=255;

    FmcjProcessInstanceVectorDeallocate(&viList);
  } while (usrchoice!=255);


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  printf("\nLogging off.\n");
  rc = FmcjExecutionServiceLogoff(service);

  rc= FmcjExecutionServiceDeallocate( &service );

  printf("Deinitializing API environment.\n");
  rc = FmcjGlobalDisconnect();

  return 0;
}
