/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQ Series Workflow API Examples - Check in / check out workitems   */
/*                                                                    */
/* This sample demonstrates how workitems can be checked out of the   */
/* normal MQS workflow environment so that the data contained in the  */
/* input container can be handled externally.                         */
/**********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ItemPackage.*;
import com.ibm.workflow.api.ProcessInstancePackage.*;
import com.ibm.workflow.api.ProgramTemplatePackage.*;
import com.ibm.workflow.api.ServicePackage.*;
import com.ibm.workflow.api.WorkItemPackage.*;


// Various needed classes

import java.io.*;
import java.util.*;


public class CheckOutIn {

public static void main (String[] args)
{

  WorkItem[] wList;

  String        WIListSort="NAME ASC";
  String WIListFilter="OWNER=CURRENT_USER";

  ReadOnlyContainer    checkOutinctnr;
  ReadWriteContainer   checkOutoutctnr;
  ReadWriteContainer   checkInOutput;
  ProgramData          checkOutData;
  ImplementationData[] impData;

  int i=0, j=0, k=0;
  int threshold=20;

  String choice="";
  int usrchoice=0;
  boolean again=false;

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (args.length!=4)
  {
    System.out.println("\nUsage: java CheckOutIn <name of agent> <loc | jndi> <UserID> <Password>");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  if      (args[1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else { System.out.println("Invalid locator policy: " + args[1]);
         System.exit(-1); }
  agent.setName(args[0]);
  ExecutionService service = agent.locate("","");
  System.out.println();
  service.logon2( args[2], args[3], SessionMode.DEFAULT, AbsenceIndicator.LEAVE );
  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() + "'.");


/*-------------------------------------------------------------------*/
/* Display a workitem list and query user for action                 */
/*-------------------------------------------------------------------*/

  do                                     // Beginning of main tool loop
  {

    wList = service.queryWorkItems( WIListFilter, WIListSort, new Integer(threshold) );

    // Display as many workitems as specified with threshold

    System.out.println("\n\n#   State          Workitem name");
    System.out.println("--------------------------------");

    if (wList.length > 0 )
    {
      for( i=0; i<wList.length; i++ )
      {
        System.out.print(i);
        if (i<10) System.out.print(" ");
        System.out.print("  ");

        switch (wList[i].state().value())
        {
          case com.ibm.workflow.api.ItemPackage.ExecutionState._UNDEFINED: System.out.print("undefined    "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._READY: System.out.print("ready        "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._FINISHED: System.out.print("finished     "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._SUSPENDED: System.out.print("suspended    "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._CHECKED_OUT: System.out.print("checkedOut   "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._EXECUTED: System.out.print("executed     "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._FORCE_FINISHED: System.out.print("forceFinished"); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._TERMINATING: System.out.print("terminating  "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._RUNNING: System.out.print("running      "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._TERMINATED: System.out.print("terminated   "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._DISABLED: System.out.print("disabled     "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._IN_ERROR: System.out.print("inError      "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._PLANNING: System.out.print("planning     "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._DELETED: System.out.print("deleted      "); break;
          case com.ibm.workflow.api.ItemPackage.ExecutionState._SUSPENDING: System.out.print("suspending   "); break;
        }
        System.out.println("  " + wList[i].name());
      }
    }
    else System.out.println("   <no workitems for user '" + args[2] + "' found>");

    // Show actions which user can perform and query user

    System.out.print("\nCommands: [o] Check out workitem            [v] View workitem info\n"
                     + "          [i] Check in workitem             [q] Quit program\n"
                     + "[o/i/v/q]? ");

    choice = in.readLine();

    usrchoice=0;

    if (choice.equalsIgnoreCase("o")) usrchoice=1;
    if (choice.equalsIgnoreCase("i")) usrchoice=2;
    if (choice.equalsIgnoreCase("v")) usrchoice=3;
    if (choice.equalsIgnoreCase("q")) usrchoice=255;

    if (wList.length>0)
    switch (usrchoice)
    {
      case 1:
/*-------------------------------------------------------------------*/
/* Check out a workitem.                                             */
/* In order to do that, the associated process instance must be in   */
/* the RUNNING state, the workitem must be in the READY state and    */
/* must be implemented by a program. In this example, the            */
/* implementation data for the Windows NT platform is queried.       */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(wList.length-1) && wList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(wList.length-1) + ".");
        }
        else
        {
          try
          {
            checkOutData = wList[i].checkOut2( ProgramRetrieval.SPECIFIED_DEFINITIONS,
                                          Basis.WINDOWS_NT );

       // Now it's possible to get the input and output container
            // from the FmcjProgramData object

       checkOutinctnr  = checkOutData.inContainer();
       checkOutoutctnr = checkOutData.outContainer();

         impData = checkOutData.implementations();

         System.out.println("Workitem has been checked out.\n" +
                            "Input Container name : " +
                          checkOutinctnr.type() + "\n" +
                            "Output Container name: " +
                            checkOutoutctnr.type() + "\n" +
                          "Execution user       : " +
                            checkOutData.executionUser() + "\n\n");
         System.out.println("Windows NT Implementation Data\n" +
                            "------------------------------\n");

       DllOptions      dllOptions;
       ExeOptions      exeOptions;
       ExternalOptions externalOptions;

       switch (impData[0].options.discriminator().value())
         {
         case com.ibm.workflow.api.ProgramTemplatePackage.Type._DLL:

           // Fetch DLL implementation data

           dllOptions = impData[0].options.dllOptions();

           System.out.println("Path and file name: " +
                              dllOptions.pathAndFileName + "\n" +
                              "Entry point name  : " +
                              dllOptions.entryPointName);

           break;

            case com.ibm.workflow.api.ProgramTemplatePackage.Type._EXE:

              // Fetch EXE implementation data

              exeOptions = impData[0].options.exeOptions();

              System.out.println("Path and file name: " +
                               exeOptions.pathAndFileName + "\n" +
                               "Working directory : " +
                               exeOptions.workingDirectoryName);
            break;

            case com.ibm.workflow.api.ProgramTemplatePackage.Type._EXTERNAL:

            // Fetch external implementation data

              externalOptions = impData[0].options.externalOptions();

              System.out.println("Executable name: " +
                                    externalOptions.executableName + "\n" +
                               "Invocation type: " +
                               externalOptions.invocationType);
            break;
       }
     } // end of try: Checkout workitem

     catch (FmcException e)
     {
       System.out.println("Checking out workitem failed with RC "
                          + e.rc + " - " + e.messageText);
       System.out.println("  StackTrace : ");
       e.printStackTrace();
     }
   }
   break;


      case 2:
/*-------------------------------------------------------------------*/
/* Check in a workitem                                               */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(wList.length-1) && wList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(wList.length-1) + ".");
        }
        else
        {
     try
     {
       do
       {
           again=false;
         System.out.print("Enter desired RC of workitem: ");
         choice = in.readLine();
         try { j = Integer.parseInt(choice); }
         catch (NumberFormatException e) { again=true; }
          } while (again);

       System.out.println();

            // Get ReadWriteContainer from workitem

            checkInOutput = wList[i].outContainer();

            // Check in workitem

            wList[i].checkIn( checkInOutput, j );

     } // end of try: Checkin workitem

     catch (FmcException e)
     {
       System.out.println("Checking in workitem failed with RC "
                          + e.rc + " - " + e.messageText);
       System.out.println("  StackTrace : ");
       e.printStackTrace();
     }
  }

  break;


      case 3:
/*-------------------------------------------------------------------*/
/* Show some workitem info                                           */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(wList.length-1) && wList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(wList.length-1) + ".");
        }
        else
        {

          System.out.println("Workitem name         : " +
                        wList[i].name() + "\n" +
                        "Process instance name : " +
                        wList[i].processInstanceName() + "\n" +
                        "Creation time         : " +
                        wList[i].creationTime().getTime().toString() + "\n" +
                        "Program implementation: " +
                        wList[i].implementation());

          FmcError error = wList[i].errorReason();
          if (error!=null)
          System.out.println("\nError RC  : " + error.rc + "\n" +
                        "Error text: " + error.messageText);

   }

        break;


  } // end switch
  else System.out.println("\nNo workitems available!");

 } while ( (usrchoice!=255) ); // end of main tool loop


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");
  service.logoff();
  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of CheckOutIn
