/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQS Workflow API Examples - Handling workitems and notifications   */
/*                                                                    */
/* This program shows how to handle workitems and notifications.      */
/**********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ItemPackage.*;
import com.ibm.workflow.api.ServicePackage.*;

// Various needed classes

import java.io.*;


public class Workitems_Notifications {

public static void main (String[] args)
{

  int i = 0;
  int j = 0;

  String choice ="";
  String ctrChoice ="N";
  int usrchoice = 0;
  String[] info;

  FmcError error;
  boolean PEAstarted = true;
  boolean again = false;

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

  // Variables for handling workitems
  WorkItem workitem;
  WorkItem[] vwList;
  String listName       ="";
  String workitemName ="";
  String WIListSort     ="";
  String WIListFilter   ="";
  ReadOnlyContainer input;
  ReadWriteContainer output;

  // Variable for handling process instances notifications
  ProcessInstanceNotification piNot;
  ProcessInstanceNotification[] piNotList;
  String piNotSort      ="";
  String piNotFilter    ="";
  String piNotName      ="";

  // Variables for handling activity instance notifications
  ActivityInstanceNotification aiNot;
  ActivityInstanceNotification[] aiNotList;
  String aiNotSort      ="";
  String aiNotFilter    ="";
  String aiNotName      ="";


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (args.length!=4)
  {
    System.out.println("\nUsage: java Workitems_Notifications <name of agent> <loc|jndi> <UserID> <Password>");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  if      (args[1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else { System.out.println("Invalid locator policy: " + args[1]);
         System.exit(-1); }
  agent.setName(args[0]);
  ExecutionService service = agent.locate("","");
  System.out.println();
  service.logon2( args[2], args[3], SessionMode.DEFAULT, AbsenceIndicator.LEAVE );
  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() + "'.");

/*-------------------------------------------------------------------*/
/* Start a PEA to permit starting workitems                          */
/*-------------------------------------------------------------------*/

  System.out.println("\nStarting PEA for user '" + service.userID() + "'... ");
  try { service.programExecutionAgentStartUp(); }
  catch (FmcException e)
  {
    if ( e.rc==FmcException.FMC_ERROR_ALREADY_STARTED )
    {
      System.out.println("PEA is already started.");
      PEAstarted=false;
    }
  }

/*-------------------------------------------------------------------*/
/* Based on user's choice, display available workitems, process      */
/* instance notifications or activity instance notifications.        */
/*-------------------------------------------------------------------*/

  do                                    // Beginning of main loop
  {
    // Query workitems and notifications from ExecutionService
    // Set workitem sorting criterium
    WIListSort   = "NAME ASC";
    // Set workitem filter criteria: only those workitems which the
    // logged-on user owns will be displayed.
    WIListFilter = "OWNER=CURRENT_USER";
    vwList = service.queryWorkItems( WIListFilter, WIListSort, null );
    // Set process instance notification sorting criterium
    piNotSort   = "NAME ASC";
    piNotList = service.queryProcessInstanceNotifications( "", piNotSort, null );
    // Set activity instance notification sorting criterium
    aiNotSort   = "NAME ASC";
    aiNotList = service.queryActivityInstanceNotifications( "", aiNotSort, null );

    System.out.print("\nDo you want to view the available [w]orkitems?\n"
         + "                                  [p]rocess instance notifications?\n"
         + "                                  [a]ctivity instance notifications?\n"
         + "[w/p/a]? ");

    choice = in.readLine();

    if (choice.equalsIgnoreCase("w"))
    {
      System.out.println("\n                   Support\n"
           + "#   State          Tool?    Workitem name\n"
           + "-----------------------------------------");
      if (vwList.length > 0 )
      {
        for( i=0; i<vwList.length; i++ )
        {
          System.out.print(i);
          if (i<10) System.out.print(" ");
          System.out.print("  ");
          switch (vwList[i].state().value())
          {
            case com.ibm.workflow.api.ItemPackage.ExecutionState._UNDEFINED: System.out.print("undefined    "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._READY: System.out.print("ready        "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._FINISHED: System.out.print("finished     "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._SUSPENDED: System.out.print("suspended    "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._CHECKED_OUT: System.out.print("checkedOut   "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._EXECUTED: System.out.print("executed     "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._FORCE_FINISHED: System.out.print("forceFinished"); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._TERMINATING: System.out.print("terminating  "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._RUNNING: System.out.print("running      "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._TERMINATED: System.out.print("terminated   "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._DISABLED: System.out.print("disabled     "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._IN_ERROR: System.out.print("inError      "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._PLANNING: System.out.print("planning     "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._DELETED: System.out.print("deleted      "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._SUSPENDING: System.out.print("suspending   "); break;
            case com.ibm.workflow.api.ItemPackage.ExecutionState._EXPIRED: System.out.print("expired      "); break;
          }
          System.out.print("  ");
            if (vwList[i].supportTools()==null) System.out.print("no ");
               else     System.out.print("yes");
          System.out.println("    " + vwList[i].name());
        }
      }
      else System.out.println("   <no workitems for user '" + service.userID() + "' found>");
      // Show actions which user can perform
      System.out.print("Commands: [v] View detailed workitem info       [i] Finish workitem\n"
           + "          [s] Start workitem                    [c] Force finish workitem\n"
           + "          [r] Restart workitem                  [o] Start support tool\n"
           + "          [f] Force restart workitem            [n] Reload a list\n"
           + "          [t] Terminate workitem                [q] Quit program\n"
           + "          [d] Delete workitem                                   \n"
           + "[v/s/r/f/t/i/c/o/n/q/d]? ");
    } /* end if: display workitems */

    if (choice.equalsIgnoreCase("p"))
    {
      System.out.println("\n#   Creation Time     Notification for");
      System.out.println("--------------------------------------");
      if (piNotList.length > 0 )
      {
        for( i=0; i<piNotList.length; i++ )
        {
          System.out.print(i);
          if (i<10) System.out.print(" ");
          System.out.println("  " + piNotList[i].creationTime().getTime().toString()
               + "  " + piNotList[i].name());
        }
      }
      else System.out.println("   <no process instance notifications for user '"
                + service.userID() + "' found>");
      // Show actions which user can perform
      System.out.print("\nCommands: [p] View process instance              [n] Reload a list\n"
           + "              notification info                  [q] Quit program\n"
           + "[p/n/q]? ");
    } /* end if: display process instance notifications */

    if (choice.equalsIgnoreCase("a"))
    {
      System.out.println("\n#   Creation Time     Notification for");
      System.out.println("--------------------------------------");

      if ( aiNotList.length>0 )
      {
        for( i=0; i<aiNotList.length; i++ )
        {
          System.out.print(i);
          if (i<10) System.out.print(" ");
          System.out.println("  " + aiNotList[i].creationTime().getTime().toString()
               + "  " + aiNotList[i].name());
        }
      }
      else System.out.println("   <no activity instance notifications for user '"
                + service.userID() + "' found>");
      // Show actions which user can perform
      System.out.print("Commands: [a] View activity instance             [n] Reload a list\n"
           + "              notification info                  [q] Quit program\n"
           + "[a/n/q]? ");
    } /* end if: display activity instance notifications */

    choice = in.readLine();
    usrchoice=0;
    if (choice.equalsIgnoreCase("v")) usrchoice=1;
    if (choice.equalsIgnoreCase("s")) usrchoice=2;
    if (choice.equalsIgnoreCase("r")) usrchoice=3;
    if (choice.equalsIgnoreCase("f")) usrchoice=4;
    if (choice.equalsIgnoreCase("t")) usrchoice=5;
    if (choice.equalsIgnoreCase("i")) usrchoice=6;
    if (choice.equalsIgnoreCase("c")) usrchoice=7;
    if (choice.equalsIgnoreCase("p")) usrchoice=8;
    if (choice.equalsIgnoreCase("a")) usrchoice=9;
    if (choice.equalsIgnoreCase("n")) usrchoice=10;
    if (choice.equalsIgnoreCase("o")) usrchoice=11;
    if (choice.equalsIgnoreCase("d")) usrchoice=12;
    if (choice.equalsIgnoreCase("q")) usrchoice=255;

    switch (usrchoice)
    {
      case 1:
/*-------------------------------------------------------------------*/
/* Show some workitem info                                           */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];
          workitem.refresh();           // Fetch secondary properties

          info = workitem.supportTools();

          System.out.print("Workitem name         : "
               + workitem.name() + "\n"
               + "Process instance name : "
               + workitem.processInstanceName() + "\n"
               + "Creation time         : "
               + workitem.creationTime().getTime().toString() + "\n"
               + "Program implementation: "
               + workitem.implementation() + "\n"
               + "Support tools         : ");

          if (info!=null)
          for (j=0; j<info.length; j++)
          {
       if   (j!=(info.length-1)) System.out.print(info[j] + ", ");
       else      System.out.println(info[j]);
     }
     else System.out.println("<none>");

          System.out.print("\nFirst notification    : ");
          if (workitem.firstNotificationTime()==null)
          System.out.println("<not set>");
          else System.out.println(workitem.firstNotificationTime().getTime().toString());
          System.out.print("\nSecond notification   : ");
          if (workitem.secondNotificationTime()==null)
          System.out.println("<not set>");
          else System.out.println(workitem.secondNotificationTime().getTime().toString());

          info = workitem.staff();

          System.out.print("Workitem Staff        : ");

          if (info!=null)
          for (j=0; j<info.length; j++)
          {
             if   (j!=(info.length-1)) System.out.print(info[j] + ", ");
             else                      System.out.println(info[j]);
     }
     else System.out.println("<none>");

          error = workitem.errorReason();
          if (error!=null)
          System.out.println("\nError RC  : " + error.rc + "\n" +
                        "Error text: " + error.messageText);
   }
        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Start a workitem                                                  */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

          try { workitem.start(); }
          catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
        }
        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Restart workitem                                                  */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

          try { workitem.restart(); }
          catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
        }
        break;


      case 4:
/*-------------------------------------------------------------------*/
/* Force restart workitem                                            */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

          System.out.print("Do you want to provide an input container? [Y/N] ");
          System.out.println();
          ctrChoice = in.readLine();
          try {
            if (ctrChoice.equalsIgnoreCase("Y"))
              {
              /* ForceRestart with input container.                     */
              /* Obtain input container.                                */
              /* Insert code here to modify ("repair") container values */
                input = workitem.inContainer();
                workitem.forceRestart2(input);
              }
            else
              {
                workitem.forceRestart();
              }/* End if*/
          }

          catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
        }
        break;


      case 5:
/*-------------------------------------------------------------------*/
/* Terminate workitem                                                */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

          try { workitem.terminate(); }
          catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
        }
        break;


      case 6:
/*-------------------------------------------------------------------*/
/* Finish workitem                                                   */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

          try { workitem.finish(); }
          catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
        }
        break;


      case 7:
/*-------------------------------------------------------------------*/
/* Force finish workitem                                             */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

          System.out.print("Do you want to provide an output container? [Y/N] ");
          System.out.println();
          ctrChoice = in.readLine();
          try {
            if (ctrChoice.equalsIgnoreCase("Y"))
              {
              /* ForceFinish with output container.                     */
              /* Obtain output container.                               */
              /* Insert code here to modify ("repair") container values */
                output = workitem.outContainer();
                workitem.forceFinish2(output);
              }
            else
              {
                workitem.forceFinish();
              }/* End if*/
          }
          catch (FmcException e)
          { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
        }
        break;


      case 8:
/*-------------------------------------------------------------------*/
/* Show process instance notification info                           */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter notification number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(piNotList.length-1) && piNotList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(piNotList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          piNot = piNotList[i];

          System.out.println("Notification for: "
               + piNot.name() + "\n"
               + "Process instance: "
               + piNot.processInstanceName() + "\n"
               + "Creation Time   : "
               + piNot.creationTime().getTime().toString() + "\n");
        }
        break;


      case 9:
/*-------------------------------------------------------------------*/
/* Show activity instance notification info                          */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter notification number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(aiNotList.length-1) && aiNotList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(aiNotList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          aiNot = aiNotList[i];

          System.out.println("Notification for: "
               + aiNot.name() + "\n"
               + "Process instance: "
               + aiNot.processInstanceName() + "\n"
               + "Creation Time   : "
               + aiNot.creationTime().getTime().toString() + "\n");
        }
        break;


      case 11:
/*-------------------------------------------------------------------*/
/* Start a support tool corresponding to a workitem                  */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(vwList.length-1) && vwList.length>0 )
        {
          System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          workitem = vwList[i];

     if (workitem.supportTools()!=null)
     {
       info = workitem.supportTools();
            System.out.println("\n#   Support tool\n"
                               + "----------------");

       for (j=0; j<info.length; j++)
       {
         if (j<10) System.out.print(j + "   " + info[j]);
         else      System.out.print(j + "  " + info[j]);
            }

       do
       {
         again=false;
         System.out.print("\n\nPlease enter support tool number: ");
         choice = in.readLine();
         try { i = Integer.parseInt(choice); }
         catch (NumberFormatException e) { again=true; }
       } while (again);

       System.out.println();

         if (i>(info.length-1))
            {
              System.out.println("Please use range 0-" + Integer.toString(info.length-1) + ".");
            }
            else
            {
              System.out.println("Starting support tool '" + info[i] + "'.");

              try {workitem.startTool(info[i]); }
         catch (FmcException e)
         { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
            }
     }
     else System.out.println("\nNo support tools available for this workitem.");
   }
        break;


      case 12:
/*-------------------------------------------------------------------*/
/* Delete a workitem                                                 */
/*-------------------------------------------------------------------*/
   do
   {
     again=false;
          System.out.print("Please enter workitem number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

     System.out.println();

     if (i>(vwList.length-1) && vwList.length>0 )
     {
       System.out.println("Please use range 0-" + Integer.toString(vwList.length-1) + ".");
     }
     else                                   // User's choice is in range
     {
       workitem = vwList[i];
       try {
             workitem.delete(); }
       catch (FmcException e)
       { System.out.println("Failed with RC " + e.rc + " - " + e.messageText); }
     }
        break;
 } // end switch
} while ( (usrchoice!=255) ); // end of main tool loop


/*-------------------------------------------------------------------*/
/* Shutdown PEA, logoff and deinit the API environment.              */
/*-------------------------------------------------------------------*/

  if (PEAstarted)
  {
    System.out.println("\nShutting down PEA.");
    service.programExecutionAgentShutDown();
  }

  System.out.println("\nLogging off.");
  service.logoff();
  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of Workitems_Notifications


